# Update script for StepOne
# 01Jun14 v1.11
# 06Jul14 Added script for Telegraph price refresh
# 02Aug14 v1.13: Added timestamp for main data update
# 01Jan15 v1.14: updated htmlParser1 for "if len(self.epicList) == 8:" following change to Telegraph website
# 30Mar15 v1.15: Minor changes to accommodate name change from StepOne Replacement to StepOne FTSE350
# 10Nov15 v1.16: Corrected problem with getting Yahoo price (previously used EPIC instead of Yahoo symbol)
#                Fixed problem with full data import (missing header row)
#                Added column Autofit after price update
# 04Sep16 v1.17  Changed formulas in columns E,F,G,H,I,K,L to differentiate between 0 value and empty cell
# 30Jun17 v1.19  Updated Yahoo price retrieval following problems with original URL
# 02May18 v1.20  Updated for Latest and Forecast Yields in Columns AH and AI
# 03Sep18 v1.21  Modified getHtml due to not working on OpenOffice
# 01Dec18 v1.21a Modified method for locating imported files following change of Windows account
# 31May19 v1.30  Modified link to Telegraph site
# 15May19 v1/40  Replaced Telegraph price source with London South-East


#import urllib.parse
import re
import sys
import time
import datetime
import socket
import os
import uno
import random

if sys.version[0:1] == '3':
    import urllib.request
    from html.parser import HTMLParser
else:
    import urllib2
    import urllib
    from HTMLParser import HTMLParser
try:
    import ssl
except:
    pass
import traceback

from com.sun.star.awt import Rectangle
from com.sun.star.awt import WindowDescriptor
from com.sun.star.awt.WindowClass import MODALTOP
from com.sun.star.awt.VclWindowPeerAttribute import OK, OK_CANCEL, YES_NO, YES_NO_CANCEL, RETRY_CANCEL, DEF_OK, DEF_CANCEL, DEF_RETRY, DEF_YES, DEF_NO
from com.sun.star.awt.FontWeight import BOLD, NORMAL

debugFile = 'C:/Users/ilongpa2/Documents/Alans_files/Temporary files/python_log.txt'

# Message box test for OO or LO version
# Uses either messageBoxOO4 or messageBoxLO4. Pretty much pot luck which one works, depending
# on which version of OpenOffice or LibreOffice is used
def messageBox(msgText, msgTitle):
    doc = XSCRIPTCONTEXT.getDocument()
    parentwin = doc.CurrentController.Frame.ContainerWindow

    try:
        messageBoxLO4(parentwin, msgText, msgTitle)
    except:
        messageBoxOO4(parentwin, msgText, msgTitle)
    else:
        pass

### following works with OO4 but not LO4 or OO Portable 3.2
def messageBoxOO4(ParentWin, MsgText, MsgTitle):
    #doc = XSCRIPTCONTEXT.getDocument()
    ctx = XSCRIPTCONTEXT.getComponentContext()
    MsgText = MsgText+"      OO"
    
    toolkit = ctx.getServiceManager().createInstanceWithContext("com.sun.star.awt.Toolkit", ctx)
    msgbox = toolkit.createMessageBox(ParentWin, 0, 1, MsgTitle, MsgText)
    
    msgbox.execute()
    #msgbox.dispose()

# Show a message box with the UNO based toolkit. Works with LO4 and OOP3.2
# But.... does not work with LO 4.2.3.3
def messageBoxLO4(ParentWin, MsgText, MsgTitle):
    MsgText = MsgText+"      LO"
    #describe window properties.
    aDescriptor = WindowDescriptor()
    aDescriptor.Type = 1                    # MODALTOP
    aDescriptor.WindowServiceName = "infobox"
    aDescriptor.ParentIndex = -1
    aDescriptor.Parent = ParentWin
    aDescriptor.WindowAttributes = 4194304  #MsgButton = OK
    
    tk = ParentWin.getToolkit()
    msgbox = tk.createWindow(aDescriptor)
    
    msgbox.setMessageText(MsgText)
    msgbox.setCaptionText(MsgTitle)
        
    return msgbox.execute()
    
######################################################################################
#### Get html from web site
def getHtml(myUrl):
    
    webTimeOut = 10
    attempts = 1
    html = 'No Response'
        
    socket.setdefaulttimeout(webTimeOut)    # set default timeout for web query
    
    try:
        context = ssl._create_unverified_context()
    except:
        pass

    while True:
        try:
            if sys.version[0:1] == '3':
                hdr = getHeader()

                req = urllib.request.Request(url=myUrl, headers=hdr)
                try:
                    response = urllib.request.urlopen(req, context=context)
                except:
                    response = urllib.request.urlopen(req)
                html = response.read()
                html = html.decode('1252', 'ignore')
        
                break
            else:
                req = urllib2.Request(myUrl, None, hdr)
                response = urllib2.urlopen(req)
                html = response.read()
                html = html.decode('1252', 'ignore')
                    
                break
        except Exception as e:
            if attempts == 1:
                messageBox("Error retrieving data\nTrying again in 30s\n"+ str(e), "Error")
            attempts += 1
            webTimeOut *= 2
            socket.setdefaulttimeout(webTimeOut)
            if attempts > 2:
                messageBox("Error retrieving data\nTerminating the process\n"+ str(e), "Error")
                sys.exit()
                break
            time.sleep(30)
    return html
    ######################################################################################
    ### end of getHtml
    ######################################################################################

######################################################################################
#### Get random browser header
def getHeader():
    header = ""
    hdrDict = {}
    randNum = random.randint(1,5)
        
    hdrDict[1] = {'User-Agent': 'Mozilla/5.0 AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11',
        'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
        'Accept-Charset': 'ISO-8859-1,utf-8;q=0.7,*;q=0.3',
        'Accept-Encoding': 'none',
        'Accept-Language': 'en-US,en;q=0.8',
        'Connection': 'keep-alive'}
        
    # Chrome / Win10
    hdrDict[2] = {'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/78.0.3904.108 Safari/537.36',
        'Accept':'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3',
        'Accept-Charset': 'ISO-8859-1,utf-8;q=0.7,*;q=0.3',
        'Accept-Encoding': 'none',
        'Accept-Language': 'en-GB,en;q=0.9,en-US;q=0.8',
        'Connection': 'keep-alive'}
        
    # Edge / Win10
    hdrDict[3] = {'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.102 Safari/537.36 Edge/18.18363',
        'Accept':'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
        'Accept-Charset': 'ISO-8859-1,utf-8;q=0.7,*;q=0.3',
        'Accept-Encoding': 'none',
        'Accept-Language': 'en-GB',
        'Connection': 'keep-alive'}
        
    # Firefox / Win10
    hdrDict[4] = {'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:70.0) Gecko/20100101 Firefox/70.0',
        'Accept':'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
        'Accept-Charset': 'ISO-8859-1,utf-8;q=0.7,*;q=0.3',
        'Accept-Encoding': 'none',
        'Accept-Language': 'n-US,en;q=0.5',
        'Connection': 'keep-alive'}
        
    # Chromium / Mint
    hdrDict[5] = {'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu Chromium/78.0.3904.108 Chrome/78.0.3904.108 Safari/537.36',
        'Accept':'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3',
        'Accept-Charset': 'ISO-8859-1,utf-8;q=0.7,*;q=0.3',
        'Accept-Encoding': 'none',
        'Referer': 'https://www.google.com',
        'HTTP-Upgrade-Insecure-Requests': '1',
 #       'Host': 'manytools.org',
        'Request-Method': 'GET',
        'Accept-Language': 'en-GB,en;q=0.9,en-US;q=0.8',
        'Connection': 'close'}
    
    return hdrDict[randNum]
    ######################################################################################
    ### end of getHeader
    ######################################################################################      

##################################################################################################
### create price dictionary from Yahoo html  
def createPriceDict(html):   
    priceDict = {}
    data = ''
    
    matchObj = re.search( r'.*?\[(.*?)\].*', html)
    if matchObj:
        data = matchObj.group(1)
    else:
        priceDict['noData'] = ['0','---']
        return(priceDict)
        
    while data:
        matchObj = re.search( r'.*?{(.*?)\}(.*)', data)
        if matchObj:
            symbolData = matchObj.group(1)
            symbol = ''
            price = ''
            currency = ''
            for symbolElement in symbolData.split(','):
                symbolElement = symbolElement.replace('"','')
                if 'symbol' in symbolElement:
                    symbol = (symbolElement[symbolElement.index(':')+1:])
                elif 'regularMarketPrice' in symbolElement:
                    price = (symbolElement[symbolElement.index(':')+1:])
                elif 'currency' in symbolElement:
                    currency = (symbolElement[symbolElement.index(':')+1:])
 
            priceDict[symbol] = [price, currency]
            
            data = matchObj.group(2)
    
    return(priceDict)
    
    ### end of createPriceDict
    ##################################################################################################    
    

# get document parameters
def docParam():
    oDoc = XSCRIPTCONTEXT.getDocument()
    oSheets = oDoc.getSheets()
    ooLocale = oDoc.getPropertyValue("CharLocale")
    return (oDoc, oSheets, ooLocale)
    
# set up status indicator
def statusIndicator(oDoc, myText, maxValue, actValue, reset):
    Controller = oDoc.getCurrentController()
    DocFrame = Controller.getFrame()
    oStatusIndicator = DocFrame.createStatusIndicator() 
    oStatusIndicator.start( myText, maxValue )
    oStatusIndicator.setValue(actValue)
    if reset: 
        oStatusIndicator.reset()
        oLayout = oDoc.getCurrentController().getFrame().LayoutManager
        StatusURL = "private:resource/statusbar/statusbar"
        oLayout.destroyElement(StatusURL)
        oLayout.createElement(StatusURL)
    
# paste data into row of sharedata sheet
def pasteShareRow(sharedataSheet, myData, shareRow):
    sharedataSheet.getCellByPosition(0,shareRow).String   = myData[0]       #name
    sharedataSheet.getCellByPosition(1,shareRow).String   = myData[1]       #isin
    sharedataSheet.getCellByPosition(2,shareRow).String   = myData[2]       #epic
    sharedataSheet.getCellByPosition(4,shareRow).Formula  = '=IF(AND(S' + str(shareRow+1) + '<>0;D' + str(shareRow+1) + '>0);D' + str(shareRow+1) + '/S' + str(shareRow+1) + ';"")' #h p/e
    sharedataSheet.getCellByPosition(5,shareRow).Formula  = '=IF(AND(U' + str(shareRow+1) + '<>0;D' + str(shareRow+1) + '>0);D' + str(shareRow+1) + '/U' + str(shareRow+1) + ';"")' #f p/e
    sharedataSheet.getCellByPosition(6,shareRow).Formula  = '=IF(AND(D' + str(shareRow+1) + '<>0;Y' + str(shareRow+1) + '>0);Y' + str(shareRow+1) + '/D' + str(shareRow+1) + ';"")' #H yield
    sharedataSheet.getCellByPosition(7,shareRow).Formula  = '=IF(AND(D' + str(shareRow+1) + '<>0;AB' + str(shareRow+1) + '>0);AB' + str(shareRow+1) + '/D' + str(shareRow+1) + ';"")'   #f yield
    sharedataSheet.getCellByPosition(8,shareRow).Formula  = '=IF(AC' + str(shareRow+1) + '=AD' + str(shareRow+1) + ';0;O' + str(shareRow+1) + '/(AC' + str(shareRow+1) + '-AD' + str(shareRow+1) + '))' #ptbv
    if myData[9]: sharedataSheet.getCellByPosition(9,shareRow).Value    = myData[9]     #gearing
    sharedataSheet.getCellByPosition(10,shareRow).Formula = '=IF(AND(S' + str(shareRow+1) + '>0;U' + str(shareRow+1) + '>0); U' + str(shareRow+1) + '/S' + str(shareRow+1) + ' - 1;"")' #eps growth
    sharedataSheet.getCellByPosition(11,shareRow).Formula = '=IF(AND(AB' + str(shareRow+1) + '<>0;U' + str(shareRow+1) + '>0);U' + str(shareRow+1) + '/AB' + str(shareRow+1) + ';"")'   #f cover
    sharedataSheet.getCellByPosition(12,shareRow).Formula = '=IF(AND(AF' + str(shareRow+1) + '<>0;O' + str(shareRow+1) + '>0);O' + str(shareRow+1) + '/AF' + str(shareRow+1) + ';"")'   #psr
    sharedataSheet.getCellByPosition(13,shareRow).String  = myData[13]  #incr div
    sharedataSheet.getCellByPosition(14,shareRow).Formula = '=D' + str(shareRow+1) + '*AG' + str(shareRow+1) + '/100'   # cap
    sharedataSheet.getCellByPosition(15,shareRow).String  = myData[15]  #index
    sharedataSheet.getCellByPosition(16,shareRow).String  = myData[16]  #sector
    sharedataSheet.getCellByPosition(17,shareRow).String  = myData[17]  #last results
    if myData[18]: sharedataSheet.getCellByPosition(18,shareRow).Value   = myData[18]       #eps
    if myData[19]: sharedataSheet.getCellByPosition(19,shareRow).Value   = myData[19]       #f eps y1
    if myData[20]: sharedataSheet.getCellByPosition(20,shareRow).Value   = myData[20]       #f eps y2
    if myData[21]: sharedataSheet.getCellByPosition(21,shareRow).Value   = myData[21]       #div y-4
    if myData[22]: sharedataSheet.getCellByPosition(22,shareRow).Value   = myData[22]       #div y-3
    if myData[23]: sharedataSheet.getCellByPosition(23,shareRow).Value   = myData[23]       #div y-2
    if myData[24]: sharedataSheet.getCellByPosition(24,shareRow).Value   = myData[24]       #div y-1
    if myData[25]: sharedataSheet.getCellByPosition(25,shareRow).Value   = myData[25]       #div y-0
    if myData[26]: sharedataSheet.getCellByPosition(26,shareRow).Value   = myData[26]       #f div y-0
    if myData[27]: sharedataSheet.getCellByPosition(27,shareRow).Value   = myData[27]       #f div y+1
    if myData[28]: sharedataSheet.getCellByPosition(28,shareRow).Value   = myData[28]       #book
    if myData[29]: sharedataSheet.getCellByPosition(29,shareRow).Value   = myData[29]       #intang
    if myData[30]: sharedataSheet.getCellByPosition(30,shareRow).Value   = myData[30]       #cash
    if myData[31]: sharedataSheet.getCellByPosition(31,shareRow).Value   = myData[31]       #turnover
    if myData[32]: sharedataSheet.getCellByPosition(32,shareRow).Value   = myData[32]       #shares
    if myData[33]: sharedataSheet.getCellByPosition(33,shareRow).Value   = myData[33]       #Latest Yield
    if myData[34]: sharedataSheet.getCellByPosition(34,shareRow).Value   = myData[34]       #Forecast Yield

    
# set hyperlink on company name in sharedata
def hyperlink():
    (oDoc, oSheets, ooLocale) = docParam()
    parentwin = oDoc.CurrentController.Frame.ContainerWindow
    sharedataSheet = oSheets.getByName("Sharedata")
    ftseSheet = oSheets.getByName("FTSE350")
    
    shareRow = 1
    while len(sharedataSheet.getCellByPosition(0,shareRow).String) > 0:
        epic = sharedataSheet.getCellByPosition(2,shareRow).String
        if ftseSheet.getCellByPosition(0,shareRow).String == epic:
            url = ftseSheet.getCellByPosition(11,shareRow).String
        else:
            ftseRow = 1
            while len(ftseSheet.getCellByPosition(0,ftseRow).String) > 0:
                if ftseSheet.getCellByPosition(0,ftseRow).String == epic:
                    url = ftseSheet.getCellByPosition(11,ftseRow).String
                    break
                else:
                    ftseRow += 1
            
        setHyperlink(oDoc, sharedataSheet, shareRow, url)
        # update status bar to indicate progress
        if not (shareRow % 20):
            statusIndicator(oDoc, "Processing hyperlinks", 17, shareRow/20, 0)
        shareRow += 1

    
# set hyperlink on cell
def setHyperlink(oDoc, sharedataSheet, shareRow, url):
    urlCell = sharedataSheet.getCellByPosition(0,shareRow)
    str = urlCell.getString()

    fld = oDoc.createInstance("com.sun.star.text.TextField.URL")
    fld.setPropertyValue("URL", url);
    fld.setPropertyValue("Representation", str);

    urlCell.setString("")

    txt = urlCell.getText()
    txt.insertTextContent(txt.createTextCursor(), fld, False)
    
###################################################################################
### start of htmlParser1 to get prices from Telegraph
class htmlParser1(HTMLParser):

    def __init__(self):
        HTMLParser.__init__(self)
        self.in_a = False
        self.in_tr = False
        self.in_tbody = False
        self.in_quote = False
        self.epicList = []
        self.priceDict = {}


    def handle_starttag(self, tag, attrs):
        
        if tag == 'a' :
            self.in_a = True
            if self.in_tbody and len(attrs) == 1 and str(attrs[0][1]).startswith('/quote/?epic'):
                self.in_quote = True
            
        if tag == 'tr' :
            self.in_tr = True
        if tag == 'tbody' :
            self.in_tbody = True


    def handle_data(self, data):
        if self.in_tbody and self.in_quote:
            data = data.strip()
            if data:
                self.epicList.append(data)
        
            
    def handle_endtag(self, tag):
        if tag == 'a':
            self.in_a = False
            
        if tag == 'tr':
            self.in_tr = False
            if len(self.epicList) == 7:
                self.priceDict[self.epicList[0]] = self.epicList[2]
                #print(self.epicList)
            self.epicList = []
        if tag == 'tbody' :
            self.in_tbody = False
            self.in_quote = False
            
    
### end of htmlParser1 to get prices from Telegraph
###################################################################################


#######################################################
### start of lseParser to get prices from lse.co.uk
### 
class lseParser(HTMLParser):
    
    def __init__(self):
        HTMLParser.__init__(self)
        self.in_tr = False
        self.in_td = False
        self.in_a = False
        self.in_table = False
        self.rowList = []
        self.priceDict = {}
        self.epic = ''
        
    def handle_starttag(self, tag, attrs):
        if tag == 'a' :
            self.in_a = True
        if len(attrs) == 4:
            if attrs[2][0] == 'data-tidm':
                self.epic = attrs[2][1]

        if tag == 'tr' :
            self.in_tr = True
        if tag == 'td' :
            self.in_td = True
        if tag == 'table' :
            self.in_table = True

    def handle_data(self, data):
        data = data.strip()
        data = data.replace(',','')
        if self.in_table and self.epic:
            self.rowList.append(data)

            
    def handle_endtag(self, tag):
        if tag == 'tr':
            self.in_tr = False
            try:
                self.priceDict[self.epic] = self.rowList[1]
            except:
                self.priceDict[self.epic] = ''
            self.rowList = []
            self.epic = ''
        if tag == 'a':
            self.in_a = False
        if tag == 'td':
            self.in_td = False
        if tag == 'table' :
            self.in_table = False

### end of lseParser to get prices from lse.co.uk
#######################################################

    
###################################################################################
### start of function to add timestamp to Help sheet

def timeStamp():
    (oDoc, oSheets, ooLocale) = docParam()
    helpSheet = oSheets.getByName("Help")
    
    myDate = time.strftime("%d%b%y %H:%M")
    helpSheet.getCellByPosition(2,2).String = "Last full data import: " + myDate

### end of function to add timestamp
###################################################################################

    
#################################################
### start of getting price from Telegraph
### 
def telegraphFunc():

    global retryCount
    priceDict = {}
    retryCount = 5
    webTimeOut = 30

    url = 'http://sharestobeclosed.telegraph.co.uk/indices/?index=NMX'

    #req = urllib.request.Request(url=url, headers={'User-Agent' : "Magic Browser"})
    
    for retries in range(retryCount):
            try:
                if sys.version[0:1] == '3':
                    response = urllib.request.urlopen(url)
                else:
                    response = urllib.urlopen(url)
                #response = urllib.request.urlopen(req, timeout = webTimeOut)
                break
            except:
                print("Connection failed", retries)
                time.sleep(5)
        
    html = response.read()
    html = html.decode('1252','ignore')
    html = html.replace('&','')
    html = html.replace(',','')
        
    p = htmlParser1()
    p.feed(html)
    priceDict = p.priceDict
    

    return [priceDict]

### end of getting price from Telegraph
###################################################


#################################################
### start of getting price from London South-East
### 
def lseFunc():

    global retryCount
    priceDict = {}
    retryCount = 5
    webTimeOut = 30

    url = 'https://www.lse.co.uk/share-prices/indices/ftse-350/constituents.html'

    html = getHtml(url)
        
    #html = html.replace('&','')
    #html = html.replace(',','')
        
    p = lseParser()
    p.feed(html)
    priceDict = p.priceDict

    return [priceDict]

### end of getting price from London South-East
########################################################################



#################################################
########################################################################
#### start of main routines#############################################
########################################################################

def updateYahooPrice():
    (oDoc, oSheets, ooLocale) = docParam()
    parentwin = oDoc.CurrentController.Frame.ContainerWindow
    sharedataSheet = oSheets.getByName("Sharedata")
    ftse350Sheet = oSheets.getByName("FTSE350")
    
    priceDict = {}
    symbolString = ''
    
    
    # Build dict of epics and yahoo symbols
    symbolDict = {}         # symbolDict[epic] = yahooSymbol
    yahooToEpicDict = {}    # yahooToEpicDict[yahooSymbol] = epic
    stockRow = 1
    while True:
        if len(ftse350Sheet.getCellByPosition(0,stockRow).String) == 0:
            break
        symbolDict[ftse350Sheet.getCellByPosition(0,stockRow).String] = ftse350Sheet.getCellByPosition(8,stockRow).String
        yahooToEpicDict[ftse350Sheet.getCellByPosition(8,stockRow).String] = ftse350Sheet.getCellByPosition(0,stockRow).String
        symbolString = symbolString + ftse350Sheet.getCellByPosition(8,stockRow).String + ','
        stockRow += 1
    
    symbolString = symbolString[:-1]        # remove trailing comma
        
    url = 'http://query1.finance.yahoo.com/v7/finance/quote?symbols='+symbolString
    
    #with open('C:/Users/User/Downloads/pydump.txt','a') as logfile:
    #    logfile.write(url)
    
    html = getHtml(url)    
    
    priceDict = createPriceDict(html)

    for x in priceDict:
        if priceDict[x][1] == 'GBP':
            priceDict[x][0] = 100 * float(priceDict[x][0])
    
#    for x in priceDict:
#        with open(debugFile,'a') as logfile:      
#            logfile.write(x + "\n")
        
    # write prices to Sharedata sheet
    stockRow = 1
    while True:
        if len(sharedataSheet.getCellByPosition(0,stockRow).String) == 0:
            break
        epic = sharedataSheet.getCellByPosition(2,stockRow).String
        #price = priceDict[symbolDict[epic]][0]
        sharedataSheet.getCellByPosition(3,stockRow).Value = float(priceDict[symbolDict[epic]][0])
        stockRow += 1
        
    
    autoFitColumns = sharedataSheet.getCellRangeByName("A1:AG1").Columns
    autoFitColumns.OptimalWidth = True
    
    oCellRangeByName = sharedataSheet.getCellRangeByName("A2")
    oDoc.CurrentController.select(oCellRangeByName)
    
    messageBox("Processing finished", "Status")
    
#===================================================================================
# import ftse350
def importFtse350():
    (oDoc, oSheets, ooLocale) = docParam()
    parentwin = oDoc.CurrentController.Frame.ContainerWindow
    ftseSheet = oSheets.getByName("FTSE350")
    
    myurl = oDoc.URL
    syspath = uno.fileUrlToSystemPath( myurl )
    directory = os.path.dirname( syspath )
    
    fullFilePath = os.path.join(directory,'ftse350.xls')
    
    #clear cells in ftse350 sheet
    ftseRow = 1
    while len(ftseSheet.getCellByPosition(0,ftseRow).String) > 0:
        ftseSheet.getCellRangeByName('A2:M400').clearContents(23)
        ftseRow += 1
    statusIndicator(oDoc, "FTSE Cells cleared", 10, 5, 0)
    
    # read the input file and write to sharedata sheet
    try:
        f = open(fullFilePath)
    except:
        messageBox("File " + fullFilePath + " not found","Error")
        return
    
    cntLine = 1
    ftseRow = 1
    myData = []
    line =f.readline()  # discard header row
    while True:
        line =f.readline()
        if len(line) == 0: # Zero length indicates EOF
            break
        #line = line.rstrip()
        myData = line.split("\t")
        
        ftseSheet.getCellByPosition(0,ftseRow).String  = myData[0]      #epic
        ftseSheet.getCellByPosition(1,ftseRow).String  = myData[1]      #name
        ftseSheet.getCellByPosition(2,ftseRow).String  = myData[2]      #index
        ftseSheet.getCellByPosition(3,ftseRow).String  = myData[3]      #isin
        ftseSheet.getCellByPosition(4,ftseRow).String  = myData[4]      #segment
        ftseSheet.getCellByPosition(5,ftseRow).String  = myData[5]      #sector
        ftseSheet.getCellByPosition(6,ftseRow).String  = myData[6]      #subsector
        ftseSheet.getCellByPosition(7,ftseRow).String  = myData[7]      #mstar id
        ftseSheet.getCellByPosition(8,ftseRow).String  = myData[8]      #yahoo
        ftseSheet.getCellByPosition(9,ftseRow).String  = myData[9]      #yahoo name
        ftseSheet.getCellByPosition(10,ftseRow).String  = myData[10]    #fund
        ftseSheet.getCellByPosition(11,ftseRow).String  = myData[11]    #url
        ftseSheet.getCellByPosition(12,ftseRow).String  = myData[12].rstrip()   #currency
        
        cntLine += 1
        ftseRow += 1
        if ftseRow == 500:
            break
        # update status bar to indicate progress
        if not (cntLine % 20):
            statusIndicator(oDoc, "Processing FTSE350", 17, cntLine/20, 0)
    
    
    f.close() # close the file
    
    statusIndicator(oDoc, "Finished", 10, 0, 1)
    #messageBox("FTSE350 processing finished", "Status")
    
#===================================================================================
# Import base data
def dataImport():
    (oDoc, oSheets, ooLocale) = docParam()
    parentwin = oDoc.CurrentController.Frame.ContainerWindow
    sharedataSheet = oSheets.getByName("Sharedata")
    
    myurl = oDoc.URL
    syspath = uno.fileUrlToSystemPath( myurl )
    directory = os.path.dirname( syspath )
    
    fullFilePath = os.path.join(directory,'stepone_FTSE350_base_oo.xls')
    
    #clear cells in sharedata sheet
    sharedataSheet.getCellRangeByName('A1:AI360').clearContents(23)
    statusIndicator(oDoc, "Cells cleared", 10, 5, 0)
    
    # read the input file and write to sharedata sheet
    try:
        f = open(fullFilePath)
    except:
        messageBox("File " + fullFilePath + " not found","Error")
        return
        
    # handle the header
    myData = []
    line =f.readline()
    myData = line.split("\t")
    
    colNum = 0
    while colNum < len(myData):
        sharedataSheet.getCellByPosition(colNum, 0).String = myData[colNum]
        colNum += 1
    
    # Now write the data
    cntLine = 1
    shareRow = 1
    
    while True:
        line =f.readline()
        if len(line) == 0: # Zero length indicates EOF
            break
        line = line.replace('"','')
        myData = line.split("\t")
        #with open('C:/Users/User/Downloads/pydump.txt','a') as logfile:
        #   logfile.write(myData[0]+"   " + str(myData[9]) + "\n")
        try:
            pasteShareRow(sharedataSheet, myData, shareRow)
        except:
            messageBox(str(shareRow), 'error')
        cntLine += 1
        shareRow += 1
        if shareRow == 500:
            break
        # update status bar to indicate progress
        if not (cntLine % 20):
            statusIndicator(oDoc, "Processing sharedata", 17, cntLine/20, 0)
        
    f.close() # close the file
    
    importFtse350()
    hyperlink()
    timeStamp()
    statusIndicator(oDoc, "Finished", 10, 0, 1)
    messageBox("Processing finished", "Status")

#===================================================================================
# Update price from the Telegraph
def updateTelegraphPrice():
    (oDoc, oSheets, ooLocale) = docParam()
    parentwin = oDoc.CurrentController.Frame.ContainerWindow
    sharedataSheet = oSheets.getByName("Sharedata")
    
    priceDict = {}
    
    [priceDict] = telegraphFunc()       # get data from web site
    
    # clear existing price data
    sharedataSheet.getCellRangeByName('D2:D370').clearContents(23)
    
    # write data to sharedata sheet
    sharedataRow = 1
    while len(sharedataSheet.getCellByPosition(0,sharedataRow).String) > 0:
        epic = sharedataSheet.getCellByPosition(2,sharedataRow).String
        if epic in priceDict.keys():
            sharedataSheet.getCellByPosition(3,sharedataRow).Value = priceDict[epic]
        if not sharedataRow % 10:       # update status bar every 10 rows
            statusIndicator(oDoc, "Processing prices", 360, sharedataRow, 0)
        sharedataRow += 1
        
    statusIndicator(oDoc, "", 0, 0, 1)      # clear up status bar
    
    autoFitColumns = sharedataSheet.getCellRangeByName("A1:AG1").Columns
    autoFitColumns.OptimalWidth = True
    
    oCellRangeByName = sharedataSheet.getCellRangeByName("A2")
    oDoc.CurrentController.select(oCellRangeByName)
    
    messageBox("Processing finished", "Status")
###
##################################################################################



#===================================================================================
# Update price from London South-East
def updateLsePrice():
    (oDoc, oSheets, ooLocale) = docParam()
    parentwin = oDoc.CurrentController.Frame.ContainerWindow
    sharedataSheet = oSheets.getByName("Sharedata")
    
    priceDict = {}
    
    [priceDict] = lseFunc()       # get data from web site
    
    # clear existing price data
    sharedataSheet.getCellRangeByName('D2:D370').clearContents(23)
    
    # write data to sharedata sheet
    sharedataRow = 1
    while len(sharedataSheet.getCellByPosition(0,sharedataRow).String) > 0:
        epic = sharedataSheet.getCellByPosition(2,sharedataRow).String
        if epic in priceDict.keys():
            sharedataSheet.getCellByPosition(3,sharedataRow).Value = priceDict[epic]
        if not sharedataRow % 10:       # update status bar every 10 rows
            statusIndicator(oDoc, "Processing prices", 360, sharedataRow, 0)
        sharedataRow += 1
        
    statusIndicator(oDoc, "", 0, 0, 1)      # clear up status bar
    
    autoFitColumns = sharedataSheet.getCellRangeByName("A1:AG1").Columns
    autoFitColumns.OptimalWidth = True
    
    oCellRangeByName = sharedataSheet.getCellRangeByName("A2")
    oDoc.CurrentController.select(oCellRangeByName)
    
    messageBox("Processing finished", "Status")
###
########################################################################


#===================================================================================
# general purpose macro for test purposes
def test():
    (oDoc, oSheets, ooLocale) = docParam()
    parentwin = oDoc.CurrentController.Frame.ContainerWindow
    sharedataSheet = oSheets.getByName("Sharedata")
    
    oCellRangeByName = sharedataSheet.getCellRangeByName("A2")
    

    oDoc.CurrentController.select(oCellRangeByName)
    
    
    
#=========================================================================
#Define which functions are visible in My Macros
g_exportedScripts = updateYahooPrice, dataImport, importFtse350, hyperlink, updateTelegraphPrice, test, updateLsePrice